/*============================================================================*/
/* eCall ANSI C fixed-point reference source code                             */
/*                                                                            */
/* File:         ecall_control.h                                              */
/* Version:      8.4.0 (Rel8) / 9.2.0 (Rel9)                                  */
/* Date:         2010-06-15                                                   */
/* Description:  control message functions (header file)                      */
/*----------------------------------------------------------------------------*/

#ifndef ECALL_CONTROL_H_
#define ECALL_CONTROL_H_

#include "ecall_defines.h"
#include "ecall_sync.h"

typedef enum {
  DlMsgNoop = -2,
  DlMsgReset,
  DlMsgStart,
  DlMsgNack,
  DlMsgAck,
  DlMsgSend,
  DlMsgHlack = SET_LLMSG
} DlData;

typedef enum {
  DlCntStart = -2,
  DlCntWait,
  DlCntNext
} DlCount;

typedef enum {
  CtrlRxIdle,
  CtrlRxSync,
  CtrlRxLock,
  CtrlTxIdle,
  CtrlTxSend
} PortState;


/*============================================================================*/
/* Control structures                                                         */
/*----------------------------------------------------------------------------*/

typedef struct {
  Int16 dlData;           /* message symbol */
  Int16 dlIndex;          /* message frame counter */
} CtrlTxPort;

typedef struct {
  Int16 dlData;           /* detected message symbol */
  Int16 dlMetric;         /* receiver metric */
} CtrlRxPort;

typedef struct {
  PortState state;        /* port state */
  Bool invert;            /* port inversion flag */
  union {
    CtrlTxPort tx;        /* port control transmitter */
    CtrlRxPort rx;        /* port control receiver */
  } u;
  const char *owner;      /* port owner identification */
} CtrlPort;

typedef struct {
  CtrlPort port;          /* port struct */
} CtrlTxData;

typedef struct {
  CtrlPort port;          /* port struct */

  SyncState *sync;        /* pointer to sync struct */
  Int16 *buffer;          /* pointer to control receiver buffer */

  Ord8  dlHlackData;      /* higher-layer message (4 bits) */
  Tern  dlRead;           /* sync indication (ternary variable) */
  Int16 dlIndex;          /* internal frame counter */
  Int16 dlSyncLock;       /* number of sync events required */
} CtrlRxData;


/*============================================================================*/
/* Control functions                                                          */
/*----------------------------------------------------------------------------*/

void CtrlSync(CtrlRxData*, const Int16*);

void CtrlTxReset(CtrlTxData*, const char*);
void CtrlTxProcess(CtrlTxData*, Int16*);
void CtrlTxMod(Int16*, Int16, Int16);

void CtrlRxReset(CtrlRxData*, const char*, SyncState*, Int16*, Int16);
void CtrlRxProcess(CtrlRxData*, const Int16*);
Int16 CtrlRxDemod(const Int16*, Int16*);

#endif
