/*============================================================================*/
/* eCall ANSI C fixed-point reference source code                             */
/*                                                                            */
/* File:         ecall_sync.h                                                 */
/* Version:      8.4.0 (Rel8) / 9.2.0 (Rel9)                                  */
/* Date:         2010-06-15                                                   */
/* Description:  synchronization functions (header file)                      */
/*----------------------------------------------------------------------------*/

#ifndef ECALL_SYNC_H_
#define ECALL_SYNC_H_

#include "ecall_defines.h"

typedef struct {
  Int32 amplitude[2];   /* amplitudes (average, maximum) */

  Bool  flag;           /* indicates sync success */
  Int16 delay;          /* synchronization delay */
  Int16 npeaks;         /* number of sync peaks detected */
  Int16 trials;         /* number of sync trials */
} SyncSub;

typedef struct {
  Int32 *mem;           /* memory for sync */
  Int32 *memWakeup;     /* memory for wakeup tone detector */

  SyncSub syncPos;      /* regular sync (non-inverted) */
  SyncSub syncNeg;      /* inverted sync */

  Int32 amplitude[3];   /* amplitudes (average, maximum, memory) */
  Int32 corrCheck[4];   /* sync check correlation values */
  Int16 corrIndex[4];   /* sync check positions */
  Int16 peakPos[4];     /* sync peak positions within control message */

  Bool  flag;           /* indicates sync success */
  Bool  invert;         /* indicates sync inversion */
  Int16 delay;          /* synchronization delay */
  Int16 delayMem;       /* synchronization delay (memory) */
  Int16 npeaks;         /* number of sync peaks detected */
  Int16 npeaksMem;      /* number of sync peaks detected (memory) */
  Int16 trials;         /* number of sync trials */
  Int16 events;         /* number of subsequent equal sync events */

  Tern  check;          /* indicates sync check result (ternary variable) */
  Int16 checkCnt;       /* counter for subsequent sync check failures */
  Int16 index;          /* frame reference for sync evaluation */
  Int16 offset;         /* frame offset */
} SyncState;


/*============================================================================*/
/* Sync functions                                                             */
/*----------------------------------------------------------------------------*/

void SyncReset(SyncState*, Int32*, Int32*);
void SyncSubReset(SyncSub*);

void Sync(SyncState*, const Int16*, const char*, Bool);
void SyncCheck(SyncState*, const Int16*, const char*);

void SyncSubPut(SyncState*, SyncSub*);
void SyncSubGet(SyncState*, SyncSub*);
void SyncSubRun(SyncSub*, const char*,
                const Int32*, const Int32*, const Int32*, const Int32*);

Int16 UpdatePeak(const Int32*, const Int32*, Int16);

void CheckPosPeaks(SyncSub*, const char*, const Int32*,
                   Int16, Int16, Int16, Int16, Int16);
void CheckNegPeaks(SyncSub*, const char*, const Int32*,
                   Int16, Int16, Int16, Int16, Int16);

void ToneDetect(SyncState*, const Int16*);

#endif
