/*============================================================================*/
/* eCall ANSI C fixed-point reference source code                             */
/*                                                                            */
/* File:         modemx.h                                                     */
/* Version:      8.4.0 (Rel8) / 9.2.0 (Rel9)                                  */
/* Date:         2010-06-15                                                   */
/* Description:  IVS and PSAP structures and functions (header file)          */
/*----------------------------------------------------------------------------*/

#ifndef MODEMX_H_
#define MODEMX_H_

#include "ecall_defines.h"
#include "ecall_control.h"
#include "ecall_modem.h"
#include "ecall_sync.h"


/*============================================================================*/
/* IVS declarations                                                           */
/*----------------------------------------------------------------------------*/

void IvsReset(const Ord8 *msd, int length);
void IvsProcess(Int16 *pcm);
void IvsSendStart(void);
void IvsReceiveAck(void);
void IvsReceiveHlack(const Ord8 data);

void IvsRxReset(void);
void IvsRxProcess(const Int16 *pcm);

void IvsTxReset(const Ord8 *msd, int length);
void IvsTxProcess(Int16 *pcm);
void IvsTxState(Int16 msg, Int16 metric, Bool syncLock);

/*----------------------------------------------------------------------------*/

typedef enum {
  IvsIdle,
  IvsTrigger,
  IvsStart,
  IvsSendMsd,
  IvsAck
} IvsState;

typedef struct {
  CtrlTxData ctrl;        /* IVS control struct */
  ModState   mod;         /* IVS modulator struct */

  Int16 state;            /* transmitter state */
  Int16 stateCntNack;     /* global NACK counter */
  Bool  startPending;     /* indicates pending START message */

  Int16 rv;               /* redundancy version */
  Int16 ulN;              /* uplink number of frames */
  Int16 ulIndex;          /* uplink frame counter */
  Int16 ulDelay;          /* uplink transmit offset in samples */

  Int16 stateCnt[SET_LLMSG+1];
  Int16 stateIgn[SET_LLMSG+1];

  Ord1  memCode[NRB_CODE_BUFFER];
  Int16 memDelay[2*PCM_LENGTH];
} IvsTxData;

typedef struct {
  CtrlRxData ctrl;        /* IVS control struct */
  SyncState  sync;        /* IVS sync struct */

  Int16 state;            /* receiver state */
  Int16 dlData;           /* downlink message symbol */
  Int16 dlIndex;          /* donwlink frame counter */
  Int16 dlMsgCnt;         /* downlink message counter */

  Int16 memCtrl[NRF_MEMCTRL*PCM_LENGTH];
  Int32 memSync[NRS_MEMSYNC];
} IvsRxData;

typedef struct {
  IvsRxData rx;           /* IVS receiver struct */
  IvsTxData tx;           /* IVS transmitter struct */
} IvsData;

IvsData* IvsGetData(void);


/*============================================================================*/
/* PSAP declarations                                                          */
/*----------------------------------------------------------------------------*/

void PsapReset(void);
void PsapProcess(Int16 *pcm);
void PsapSendStart(void);
void PsapSendHlack(const Ord8 data);
void PsapReceiveMsd(const Ord8 *msd, int length);

void PsapRxReset(void);
void PsapRxProcess(const Int16 *pcm);
void PsapRxUplink(const Int16 *pcm);
void PsapRxDownlink(void);

void PsapTxReset(void);
void PsapTxProcess(Int16 *pcm);

/*----------------------------------------------------------------------------*/

typedef enum {
  PsapIdle,
  PsapTrigger,
  PsapStart,
  PsapNack,
  PsapAck,
  PsapHlack
} PsapState;

typedef struct {
  CtrlRxData ctrl;        /* PSAP control struct */
  SyncState  sync;        /* PSAP sync struct */
  ModState   mod;         /* PSAP modulator struct */

  Int16 state;            /* receiver state */
  Int16 rv;               /* redundancy version */
  Int16 ulN;              /* uplink number of frames without muting */
  Int16 ulIndex;          /* uplink frame counter */
  Int16 mgIndex;          /* uplink position in muting gap table */
  Int16 ulTrials;         /* uplink decoding trails */
  Int16 ulSyncTail;       /* sync observation counter after sync success */

  Ord8  dlHlackData;      /* downlink higher-layer message (4 bits) */
  Int16 dlData;           /* downlink message symbol */
  Int16 dlIndex;          /* donwlink frame counter */
  Int16 dlMsgCnt;         /* downlink message counter */

  Ord8   *msd;            /* MSD in byte representation */
  Ord1   *msdBin;         /* MSD in binary representation */
  Int16  *memCtrl;        /* buffer for control and data demodulation */
  IntLLR *memCode;        /* soft bit buffer for decoding */

  char buffer[0
    + sizeof(IntLLR)* NRB_CODE_ARQ
    + sizeof(Int16) * NRF_MEMCTRL*PCM_LENGTH
    + sizeof(Int32) * NRS_MEMSYNC
    + sizeof(Int32) * 2*(NRF_SYNC+1)];
} PsapRxData;

typedef struct {
  CtrlTxData ctrl;        /* PSAP control struct */
} PsapTxData;

typedef struct {
  PsapRxData rx;          /* PSAP receiver struct */
  PsapTxData tx;          /* PSAP transmitter struct */
} PsapData;

PsapData* PsapGetData(void);

#endif
