/**
 * Interface for candidate IVS and PSAP modem implementations.
 *
 * @copyright 2008, research in motion ltd.
 *
 * @version $Id: //depot/main/dsp/ecall/c/testlab/modem.h#4 $
 */

#ifndef __MODEM_H__
#define __MODEM_H__

#include "rimtypes.h"

#define MSD_MAX_LENGTH (140)

/*
 * IVS reset: called at start of each test case.
 */
void IvsReset(const Ord8 *msd, int length);

/*
 * IVS request for MSD: called when IVS should trigger MSD transmission.
 */
void IvsSendStart(void);

/*
 * IVS modem processing: called for each 20ms audio frame.
 */
void IvsProcess(Int16 *pcm);

/*
 * Called by IVS implementation to indicate ACK was received.
 */
void IvsReceiveAck(void);

/*
 * Called by IVS implementation to indicate HLACK data was received.
 */
void IvsReceiveHlack(const Ord8 data);

/*
 * PSAP reset: called at start of each test case.
 */
void PsapReset(void);

/*
 * PSAP request for MSD: called when PSAP should trigger request from IVS.
 */
void PsapSendStart(void);

/*
 * PSAP to send the HLACK message.
 */
void PsapSendHlack(const Ord8 data);

/*
 * PSAP modem processing: called for each 20ms audio frame.
 */
void PsapProcess(Int16 *pcm);

/*
 * Called by PSAP implementation to indicate MSD was received.
 */
void PsapReceiveMsd(const Ord8 *msd, int length);

#endif
